# -*- coding: utf-8 -*-

import re
import datetime

import shopify
import pyactiveresource
import shopify_python_api_patcher
from shopify_python_api_patcher.utils import get_current_release

from gspread.exceptions import APIError
from tracking_numbers import get_tracking_number

from em_tasks import logger
from em_tasks.tools.shopify_orders_sheet_base import ShopifyOrdersSheetBase
from em_tasks.store.store_util import StoreUtil
from em_tasks.google.sheet_api import SheetAPI
from em_tasks.utils.order_reader import GSheetOrderReader
from em_tasks.store.models import *


class ShopifyOrdersSheetUpdater(ShopifyOrdersSheetBase):
  def __init__(self, credential, service_account_data, days):
    self.service_account_data = service_account_data
    self.sheet_api = SheetAPI(service_account_data)
    self.order_reader = GSheetOrderReader(self.sheet_api)
    self.days = days
    self.tracking_pattern = re.compile('[0-9A-Za-z-]{10,100}')

    store_name = credential['name']
    app_token = credential['token']
    release = get_current_release()

    shop_url = "{}.myshopify.com".format(store_name)
    session = shopify.Session(shop_url, release, app_token)
    shopify.ShopifyResource.activate_session(session)
    self.shop = shopify.Shop.current()

  def run(self):
    ss_ids = dict()
    for items in self.__class__.spreadsheet_ids.values():
      for spreadsheet_id in items.values():
        ss_ids[spreadsheet_id] = None

    for spreadsheet_id in ss_ids.keys():
      for worksheet, headers, orders in self.order_reader.read(spreadsheet_id, self.days):
        sheet_orders = dict()
        records = []
        for order in orders:
          order_id = order.get('order-id', '')
          if not order_id:
            continue

          order_id_valid = True
          for part in order_id.split('-'):
            if not part.isnumeric():
              order_id_valid = False
              break
          if not order_id_valid:
            continue

          remark = order.get('remark', order.get('status', '')).lower()
          if remark == 'cancelled':
            continue

          order_id = order_id.split('-').pop(0)
          if order_id not in sheet_orders:
            sheet_orders[order_id] = []

          sheet_orders[order_id].append({
            'order': order,
            'headers': headers,
            'worksheet': worksheet
          })

        oids = ','.join([str(oid) for oid in sheet_orders.keys()])
        for order in shopify.Order.find(ids=oids, status='any', limit=250):
          if not order.cancelled_at:
            continue

          oid = str(order.id)
          if oid not in sheet_orders:
            continue

          s_orders = sheet_orders[oid]
          for s_order in s_orders:
            s_order_data = s_order['order']
            headers = s_order['headers']
            worksheet = s_order['worksheet']

            try:
              remark_h = 'remark'
              if 'remark' not in headers:
                remark_h = 'status'
              sku_col = headers.index('order-id') + 1
              remark_col = headers.index(remark_h) + 1
              remark_col_offset = remark_col - sku_col

              cell = self.sheet_api.find_cell_by_key(worksheet, s_order_data['order-id'])
              row = cell.row
              r_col = cell.col + remark_col_offset
              self.sheet_api.update_worksheet_cell(worksheet, row, r_col, 'Cancelled')

              logger.info(
                '[SheetOrderUpdated] SheetName: %s, OrderID: %s, OrderNumber: %s, Status: Cancelled',
                s_order_data['sheet'], s_order_data['order-id'], s_order_data['order_number'])
            except:
              logger.warning('[SheetOrderError] SheetName: %s, OrderID: %s, OrderNumber: %s',
                s_order_data['sheet'], s_order_data['order-id'], s_order_data['order_number'])
